\name{PTBi}
\alias{PTBi}
\alias{PTBms}
\alias{PTBmsb}
\alias{PTBmsbr}
\alias{PTBn}
\alias{PTBnc}
\alias{PTBns}
\alias{PTBnbr}
\alias{PTBnbrr}
\alias{PTBnsbr}
\alias{PTBnsbrr}
\alias{PTBu}
\alias{PTBuc}
\alias{PTBus}
\alias{PTBubr}
\alias{PTBubrr}
\alias{PTBusbr}
\alias{PTBusbrr}
\title{ functions to add random noise to a vector }
\description{
These functions add noise to a vector.
These are helper functions for \code{\link{perturb}}
They are used to select the type and magnitude of
noise applied to each vector in the data frame,
when running the perturbation sensitivity analysis. Use them only 
if they are substantively justified -- you can also supply custom
functions for use with this perturbation framework.
}

\usage{
	#
	# identity function
	#

	PTBi(x,size=1) 				# identify function, returns original vector
						# size is a dummy value, it is ignored
	#
	# minimal noise
	#

	PTBms(x, size=1)				# adds minimal, scaled, noise 
	PTBmsb(x, size=1, lbound=0, ubound=1)		# adds minimal, scaled, noise,
						# resulting vector truncated to bounds
	PTBmsbr(x, size=1, lbound=0, ubound=1)		# adds minimal, scaled  noise, out of-bounds elements
						# of vector resamples until within bounds

	#
	# Normally distributed noise
	#

	PTBn(x, size = 1)			# adds normally distributed noise
	PTBnc(x, size = 1)			# normally distributed noise, 
						# recentered to guarantee mean-zero disturbance
	PTBns(x, size = 1)			# adds normally distributed, scaled, noise

	PTBnbr(x, size = 1, lbound=0, ubound=1)# 
	PTBnsbr(x, size = 1, lbound=0, ubound=1)# adds normally (scaled)  distributed noise
						# out of-bounds elements
						# of vector resampled until within bounds

	PTBnbrr(x, size = 1, lbound=0, ubound=1)# 
	PTBnsbrr(x, size = 1, lbound=0, ubound=1)	# adds normally distributed,scaled noise
						# noise rescaled to bounds, and then 
						# out of bounds elements 
						# resampled until within bounds


	#
	# Uniformly distributed noise

	PTBu(x, size = 1)			# adds uniformly distributed noise
	PTBuc(x, size = 1)			# uniformly distributed noise, 
						# recentered to guarantee mean-zero disturbance
	PTBus(x, size = 1)			# adds uniformly distributed, scaled, noise

	PTBubr(x, size = 1, lbound=0, ubound=1)# 
	PTBusbr(x, size = 1, lbound=0, ubound=1)# adds uniformly, scaled,  distributed noise
						# out of-bounds elements
						# of vector resampled until within bounds

	PTBubrr(x, size = 1, lbound=0, ubound=1)# 
	PTBusbrr(x, size = 1, lbound=0, ubound=1)	# adds unirformly distributed,scaled noise
						# noise rescaled to bounds, and then 
						# out of bounds elements 
						# resampled until within bounds


}

\arguments{ 
  \item{x}{ vector to be perturbed }
  \item{size}{ magnitude of noise }
  \item{ubound}{ upper bound, can be a scalar or a vector of the same length as x }
  \item{lbound}{ lower bound, can be a scalar or a vector of the same length as x }
}

\details{

	Noise is randomly adds to each observation in the vector according to
	the chosen distribution function:
	- 'minimal' adds .Machine$double.ep) or subtracts .Machine$double.neg.eps
	- 'uniform' adds uniformly distributed random [-size/2,+size/2] 
	- 'normal' adds Normal(mean=0,stddev=size)
			 
	Noise is assymptotically mean zero. To guarantee mean-zero noise
	for a particular sample, use the "centered" helper functions, which
	adjust the noise vector to be approximately mean zero.

	The 'scaled' functions,  multiply each random deviate by $x_i$ for 
	each observation in x. This is useful if measurement error is
	heteroscedastic.

	Some helper functions can return a noisy vector guaranteed to be
	within given upper and lower bounds. This can be done through
	truncation, through resampling out-of-bounds observations, or
	by reducing the amount of noise added to entries close to the bounds.
	
}

\value{
	Returns a new vector x', with the same length as x.
}

\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}
}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
}


\seealso{ \code{\link{perturb}}}
\examples{
	x=1:1000
	x.i = PTBi(x)
	x.m = PTBms(x)
	x.u = PTBu(x,size=.01)
	x.us = PTBus(x,size=.01)
	x.uc = PTBuc(x,size=.01)

	sum(x-x.i)		# should be 0, identity transform
	sum(which(x!=x.i))	# also 0

	sum(x-x.m)		# should be quite small, very close to 0
	sum(which(x!=x.m))	# should be near 1000

	sum(x-x.u)		# should be small
	sum(which(x!=x.m))	# should be near 1000

	sum(x-x.us)		# should be small
	sum(which(x!=x.m))	# should be near 1000

	sum(x-x.uc)		# should be near 0
	sum(which(x!=x.m))	# should be near 1000
	
}
\keyword{misc}
\keyword{optimize}
\keyword{distribution}

\eof
\name{de Haan Global Optimality Test}

\alias{dehaan}

\title{Function to test that an optimum from MLE, NLS, or other non-linear optization
routine is a global optimum}

\description{
Implements the de Haan test for identification of the global optimum of a likelihood
surface.
}


\usage{
   dehaan(llTest, llMax, pval=.05 )
}

\arguments{
  \item{llTest}{Vector of randomly generated likelihood values}
  \item{llMax}{Value of the likelihood function for a candidate global optimum}
  \item{pval}{p-value of the test}
}

\details{
\texttt{dehaan} computes a (1-p) confidence interval for the global optimum of a
likelihood surface from a vector of user-supplied randomly chosen likelihood values.  
A user-supplied candidate maximum likelihood value is evaluated against confidence
interval.
}

\value{

\texttt{dehaan=TRUE} if the candidate value is greater than the (1-p) confidence
interval for the true optimum.
}

\note{

The choice of the vector of user-supplied random likelihood values is somewhat of an
art.  The parameter space should be large enough to capture the true optimum, but not
so large as to include illegal or impractical parameter values.

The generated confidence interval depends on the number of random draws from the
parameter space.  The number should be large enough to allow the application of
asympotic theory.  A recommended number of random evaluations of the likelihood
function at a given set of parameter values is 500 or more, but depends on the size of
the parameter space.  Users are encouraged to experiment with more and less draws and
observe results.

}

\references{

Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}

de Haan, L. 1981.  ``Estimation of the Minimum of a Function Using Order Statistics.''
\emph{Journal of the American Statistical Association} {\bf 76}, 467-9.

Veall, M. R. 1990.  ``Testing for a Global Maximum in an Econometric Context.''
\emph{Econometrica} {\bf 58} 1459-65.

}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
, Michael P. Mcdonald
}

\example{

  # The deHaan test is constructed as a maximum likelihood
  # test, with negative values for the likelihood.  The BOD problem
  # is a non-linear least squares minimization problem.  This test
  # is implemented using the negative of the sum of squares for consistency
  # with the deHaan framework of maximum likelihood.

  data("BOD",package="stats")
  stval=expand.grid(A = seq(10, 100, 10), lrc = seq(.5, .8, .1))
  llfun<-function(A,lrc)
        -sum((BOD$demand - A*(1-exp(-exp(lrc)*BOD$Time)))^2)
  lls=NULL 
  for (i in 1:nrow(stval))  {
     lls = rbind(lls, llfun(stval[i,1], stval[i,2]))
  }
  fm1 <- nls(demand ~ A*(1-exp(-exp(lrc)*Time)),  
                data = BOD, start = c(A = 20, lrc = log(.35)))
  ss = -sum(resid(fm1)^2)
  dehaan(lls, ss)

}

\keyword{manip}

\keyword{misc}

\keyword{math}

\keyword{htest}


\eof
\name{Distribution Test Data, and log relative error function}
\alias{chisqtst}
\alias{ftst}
\alias{gammatst}
\alias{normtst}
\alias{ttst}
\alias{LRE}
\title{ Benchmark data to test the accuracy of statistical distribution functions, function to compute log relative error }
\description{
	This is benchmark data, used to test the accuracy of statistical distribution functions. Also included is a function for computing log-relative error, a measure of accuracy.
}
\usage{
	data(ttst)
	data(ftst)
	data(gammatst)
	data(normtst)
	data(chisqtst)
	LRE(x,correct, use.LAE=TRUE)
}
\arguments{
  \item{x}{ vector of computed values}
  \item{correct}{ vector of correct values}
  \item{use.LAE}{use log absolute error if $c_i$==0. If false, returns NA where $c_i$==0}
} 

\value{ 
        Returns a new vector of log-relative-errors (log absolute error where $c_i$ ==0).
	The resulting values are roughly interpretable as the number of significant digits
	of agreement between c and x. Larger numbers indicate that x is a relatively
	more accurate approximation of c. Numbers less than or equal to zero indicate
	complete disagreement between x and c.
}

\details{
These tests values are tab-delimited ascii datasets, with a variable
header line. Typical variables are:

        P - probablilty value
        DF - degrees of freedom
        INVDIST - inv the inverse function value for P:
                i.e.: invt(P,DF) = INVT
        INVDIST - inv the inverse function value for P:
                i.e.: invt(P,DF) = INVT
        PINVDIST - value for cumulative distribrution of
                        INVDIST
                i.e.: cumt(INVT,DF) = PINVT

Note that because of limits in numerical precision, cumt(invt(P,DF)) is not always equal to P

Of course, not all possible values of P and DF can be listed.
The test values for P  were created from systematic and random samples in 
{ [1e-12,1-1e-12],0,1 }, and [0,1E5] respectively.

We use Kneusel's ELV program (1989) to calculate the values the cumulative and 
inverse distributions. The results are claimed by Knsusel to be accurate to 6 digits. We
checked these results using Brown's (1998) DCDFLIB library. The results
of both calculations agreed to the 6 digits supplied by ELV, but in a small
number of cases, DCDFSTAT's calculation at the 7th digit indicated that
the sixth digit would change if rounded. 

Note that both ELV and DCDFLIB can generate many more distributions than
are included here. Other resources are described in the references
}

\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}

Altman, Micah and Michael McDonald, 2001. "Choosing Reliable Statistical Software." 
\emph{PS: Political Science \& Politics} 24(3): 681-8

Barry W. Brown, James Lovato, Kathy Russell, 1998,   DCDFLIB,
\url{ftp://odin.mda.uth.tmc.edu in pub/unix/dcdflib.c-1.0-tar.Z}

Kneusel, L. 1989. \emph{Computergesteutzte Berechnung statistischer Verteilungen.} Olde
nbourg, Meunchen-Wien.
\url{http://www.stat.uni-muenchen.de/~knuesel/elv/elv.html}
}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}, Michael McDonald
}


\examples{

# simple LRE examples
LRE(1.001,1) # roughly 3 significant digits agreement
LRE(1,1) # complete agreement
LRE(20,1) # complete disagreement

#
# how accurate are student's t-test functions?
#

data(ttst)
# compute t quantiles using benchmark data
tqt = qt(ttst$p,ttst$df)

# compute log-relative-error (LRE) of  qt() results, compared to 
# correct answers

lrq = LRE(tqt, ttst$invt);

# if there are entries with LRE's of < 5, there may be
# significant inaccuracies in the qt() function

table(trunc(lrq))

# now repeat process, for pt()

tpt = pt(ttst$invt,ttst$df)
lrp= LRE(tpt, ttst$pinvt);
table(trunc(lrp))


}
\keyword{misc}
\keyword{debugging}

\eof
\name{frexp}
\alias{frexp}
\title{Function to convert vector of floating-point numbers to fractional and integral components}
\description{
	The  frexp() function returns a matrix with the normalized fraction [.5,1) of the vector in the first
	column and the exponent (a power of two) in the second column. (If x is zero,
       then the normalized fraction is zero and zero is stored in the exponent.)
}
\usage{
	frexp(v)
}
\arguments{
  \item{v}{ vector of doubles }
}
\details{
  This is an R wrapper around the <math.h> frexp() function. 
}
\value{
Returns a 2xn matrix. Thethe normalized fraction [.5,1) of x is the first
	column and the exponent (a power of two) in the second column. (If x[i] is zero,
      then the normalized fraction of x[i] is zero and zero is stored in the exponent.)
}

\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}
}

\author{
Micah Altman
\email{Micah\_Altman\@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
}


\examples{
x = runif(10)
y = frexp(x)
y
# this is now true by construction
x==y[,1] *2^y[,2]
}
\keyword{manip}
\keyword{misc}
\keyword{math}

\eof
\name{perturb}
\alias{perturb}
\title{ Data Perturbations based Sensitivity Analysis }
\description{
This function replicates an statistical analysis using slightly perturbed
versions of the original input data, and then analyzes the sensitivity of the
model to susch changes.  This can be used to draw attention to
        inferences that cannot be supported confidently given the current
        data, model, and algorithm/implementation,


}
\usage{
perturb(data, statistic, ..., ptb.R = 1, ptb.ran.gen = PTBms, ptb.s = NULL)
}
\arguments{
  \item{data}{ data matrix to be perturbed }
  \item{statistic}{ statistic model to be run on data }
  \item{\dots}{ additional arguments to statistical model }
  \item{ptb.R}{ number of replications for perturbation analysis }
  \item{ptb.ran.gen}{ a single function, or a vector of functions to be
		used to perturb each vector see \code{\link{PTBi}} }
  \item{ptb.s}{ a size, or vector of sizes, to be used in the vector perturbation functions}
}
\details{

Sensitivity to numerical inaccuracy, and measurement error is very hard to 
measure formally. This empirical sensitivity tests draws 
attention to inferences that cannot be supported confidently given the current
data, model, and algorithm/implementation,

The empirical approach works by replicating
the original analysis while slightly perturbing the original  input data
in different ways. The sensitivity of the model estimates (e.g. 
estimated coefficients, standard errors and log-likelihoods) are then
summarized. 

The sensitivity analysis cannot be used to prove the accuracy of
a particular method, but is useful in drawing attention to
potential problems. Further experimentation and analysis may be
necessary to determine the specific cause of the problem:
numerical instability, statistical sensitivity to measurement error,
or ill-conditioning.

}
\value{
	Returns a list which contains the result  of each model run. Along
with attributes about the settings used in the perturbations.
	Use \code{summary} to summarize the results or extract a matrix of
 of the model parameters across the entire set of runs.
}

\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}
}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
}


\seealso{ See Also as \code{\link{PTBi}}}
\examples{

# Examine the sensitivity of the GLM from Venables & Ripley (2002, p.189)
# as described in the glm module.
# 
# Perturb the two independent variables using +/- 0.025% 
#	(relative to the size of each observations)
# uniformly distributed noise. Dependent variable is not being modified.
# 
# Summary should show that estimated coefficients are not substantively affected by noise.

data(anorexia,package="MASS")
panorexia = perturb(anorexia, glm, Postwt ~ Prewt + Treat + offset(Prewt),
     family=gaussian, 
    ptb.R=100, ptb.ran.gen=c(PTBi,PTBus,PTBus), ptb.s=c(1,.005,.005) )
summary(panorexia)

# Use classic longley dataset. The model is numerically unstable, 
# and much more sensitive to noise.  Smaller amounts of noise tremendously
# alter some of the estimated coefficients:
# 
# In this example we are not perturbing the dependent variable (employed) or 
# the year variable. So we assign then PTBi or NULL in ptb.ran.gen )

if(version$major<2) {
	data(longley, package="base")
} else {
	data(longley, package="datasets")
}

plongley = perturb(longley,lm,Employed~., ptb.R=100, 
    ptb.ran.gen=c(PTBi, replicate(5,PTBus),PTBi), ptb.s=c(1,replicate(5,.001),1))
sp=summary(plongley)

# summarizes range
print(sp)

# look in summary object to extract more ...
names(attributes(sp))

# print metrix of coefficients from all runs
coef= attr(sp,"coef.betas.m")
summary(coef)


}
\keyword{misc}
\keyword{optimize}

\eof
\name{sechol}
\alias{sechol}
\title{Schnabel-Eskow Choleksy Decomposition}
\description{Perform the Schnabel-Eskow Choleksy Matrix Decomposition}
\usage{
sechol(A, tol=.Machine$double.eps, silent=TRUE)
}
\arguments{
  \item{A}{a square matrix object which can be non-singular.}
  \item{tol}{a tolerance parameter}
  \item{silent}{print debugging messages}
}
\details{
The algorithm of Schnabel and Eskow (Schnabel, R. B. and Eskow, E. 1990.
"A New Modified Cholesky Factorization." SIAM Journal of Scientific Statistical 
Computing 11, 1136-58.) improves the Gill/Murray approach of incrementing diagonal
values of a singular matrix sufficiently that Cholesky steps can be performed. 
The algorithm is based on applying the Gerschgorin Circle Theorem to reduce the 
infinity norm of the incrementing matrix. The strategy is to calculate
delta values that reduce the \emph{overall} difference between the singular matrix and
the incremented matrix.
}
\value{A Cholesky decomposition of the matrix C+E where C is the original non-singular
matrix and E is the minimal diagonal increment according to the Gerschgorin Circle
Theorem.  If the input matrix is non-singular, then the Cholesky decomposition of C
is returned.}
\references{Schnabel, R. B. and Eskow, E. 1990.
"A New Modified Cholesky Factorization." SIAM Journal of Scientific Statistical 
Computing 11, 1136-58.

Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}

}
\author{Jeff Gill}
\seealso{
  \code{\link[MASS]{ginv}}
  \code{\link{solve}}
  \code{\link{chol}}
  \code{\link{svd}}
  \code{\link{qr}}
}
\examples{
# compare with chol() on a non-singular matrix
S <- matrix(c(2,0,2.4,0,2,0,2.4,0,3),ncol=3)
chol(S)
T <- sechol(S)
t(T) %*% T

# an example with a singular matrix
S <- matrix(c(2,0,2.5,0,2,0,2.5,0,3),ncol=3)
sechol(S) 
t(T) %*% T  
    
# another example with a singular matrix
S <- matrix(c(2,0,10,0,2,0,10,0,3),ncol=3)
T <- sechol(S)
t(T) %*% T  
}
\keyword{array}
\keyword{optimize}

\eof
\name{Starr Global Optimality Test}
\alias{starr}

\title{Function to test that an optimum from MLE, NLS, or other non-linear optization
	routine is a global optimum}

\description{ Implements the Starr test for identification of the global optimum of a
	likelihood surface. }

\usage{
 starr(betas, tol=.0001, dmethod="euclidean")
}

\arguments{
  \item{betas}{Vector of parameter values}
  \item{tol}{Tolerance distance between two parameter vectors to consider as "unique"
	optimum}
  \item{dmethod}{method used to compute distance between two parameter vectors}
}

\details{
	\texttt{starr} computes the probability that a local optimum, which may or may
not be the global optimum, of a likelihood function has not been observed.  The
probability is generated by counting the observed number of ``basins of attraction'' -
starting values that lead to an local optimum.
}

\value{ 
\texttt{starr} is a class "double" with value equal to the probability than a
	local optimum is unobserved.
}

\note{
\texttt{starr} is given a vector of \texttt{betas} parameter values identifying local optima resultant of a search algorithm, such as \texttt{mle} or \texttt{nls}.  The starting values used to generate \texttt{betas} can be a grid, or for larger n-dimenional parameter spaces, may be randomly chosen.  The parameter values identifying a local optimum are passed to \texttt{starr} for each set of starting values that lead to that local optimum.
}



\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}

Finch, S. J., Mendell, N. R., Thode, H. C., Jr. 1989. ``Probabilistic Measures of
Adequacy of a Numerical Search for a Global Maximum.'' \emph{Journal of the American
Statistical Association} {\bf 84}, 1020-3.

Starr, N. 1979. ``Linear Estimation of the Probability of Discovering a New Species.''
\emph{Annals of Statistics} {\bf 7}, 644-52.
}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
, Michael McDonald
}


\example{

	x=rbind(c(1,1,1), c(1,2,1), c(1,1.1,1), c(1,2,1), c(3,4,5));
	starr(rbind(1,1,2,2));}

}
\keyword{manip}
\keyword{misc}
\keyword{math}
\keyword{htest}

\eof
\name{trueRandom}
\alias{trueRandom}
\alias{resetSeed}
\alias{runifT}
\alias{initPool}
\title{ Function to return TRUE (not pseudo) random numbers, based on system and networked entropy collection. }
\description{
	This function makes use of hardware-generated entropy to supply 
true random numbers. Entropy collection is slow, so its primary use to
provide random numbers for cryptographic key generation and for setting 
the seed value for subsequent PRNG generation. 
}
\usage{
	runifT(n,min=0,max=1)
	resetSeed()
	initPool(size=512, hbok=TRUE, devrndok=TRUE, silent=FALSE) 
}
\arguments{
  \item{n}{ number of observations. If 'length(n) > 1', the length is taken to be the number required. }

 min,max: lower and upper limits of the distribution.
  \item{min,max}{ lower and upper limits of the distribution }
  \item{size}{ number of bytes of entropy to be retrieved into the local entropy pool}
  \item{hbok}{ use the "hotbits" entropy server }
  \item{silent}{ do not issue warnings }
  \item{devrndok}{ use /dev/random on Unix-derived systems }
}

\details{
\code{runifT} returns true random numbers in a uniform distribution.

\code{resetSeed} resets R's own PRNG seed, \code{set.seed}, supplying a true random number

\code{initPool} provides lower level control over how the external entropy sources
used by \code{runifT} and \code{resetSeed} are accessed

These routines	provides true random numbers by accessing external entropy collectors.
Currently entropy can be retrieved from two different
sources. (1) The "Hotbits" server \url{http://www.fourmilab.ch/hotbits/} supplies
random bytes based on radioactive decay. (2) On Unix systems, the the  kernel gathers
environmental noise from device drivers and other sources into an entropy  pool, which
can be accessed through '/dev/random'.

By default, /dev/random will be preferred, if available, with hotbits used only
if /dev/random is not found. In addition, entropy will be requested from the
external sources in 512byte blocks (runifT will automatically request more
blocks if necessary to satisfy a request). Use initPool to change the source and
block size.

}

\value{
\code{runifT} 	Returns a vector of uniformly distributed true random numbers.
\code{resetSeed} returns the status of \code{set.seed()}

}

\references{
Altman, M., J. Gill and M. P. McDonald.  2003.  \emph{Numerical Issues in Statistical
Computing for the Social Scientist}.  John Wiley \& Sons.
\url{http://www.hmdc.harvard.edu/numerical_issues/}
}

\author{
Micah Altman
\email{Micah\_Altman@harvard.edu}
\url{http://www.hmdc.harvard.edu/micah_altman/}
}
\warning{  This relies on external sources of true random numbers.
If a source of true random numbers is unavailable reset.seed() uses the clock value
and true.random() will return NULL.
	 }

\seealso{ See Also 
	\code{\link{runif}},
	\code{\link{.Random.seed}},
}
\examples{
# note, if you are using Windows, you must be on-line
# to get to entropy generator, will fall-back to pseudo-random
# numbers when off-line

# only needed  to pass R CMD check on online windows
w = options("warn");
options(warn=0);

# reset the seed for runif() based on a true random value

resetSeed()
y=runif(100)

# generate true random values directly (may block for long periods if 
# if entropy pool is empty)

y=runifT(1000)
ty= table(trunc(5*y))
print(ty);
chisq.test(ty)
options(warn=as.integer(w));

}
\keyword{misc}
\keyword{datagen}
\keyword{distribution}

\eof
